
/*mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
			SVGA256 GRAPHICS LIBRARY
	     Copyright (C) 1993 by Orthware.   All rights reserved.
/*mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm*/

/*************************************************************************
/* S256DEMO.C		Demonstrates the SVGA256 graphics library
/*			for SuperVga, hi-res, 256-color modes.
/*************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <dos.h>
#include <malloc.h>	/* might be <alloc.h> for some Borland Compilers */
#include <conio.h>
#include "svga256.h"
#include "s256demo.h"

/********************************
/* misc #define's for this module
/********************************/

#define TRUE		1
#define FALSE		0
#define YES		1
#define NO		0
#define BYTE		unsigned char
#define WORD		unsigned short 
#define LONG		unsigned long
#define	TICKS		(*(LONG far *) 0x0040006c)

#define FADESTEPS	10		/* number of steps in screen fading */
#define FADEDELAY	15		/* millisec delay betw fade steps */
#define IMAGEWIDTH	501		/* pixel width of the demo image */
#define IMAGEHT		333		/* pixel height of the demo image */
#define NO_ZOOM		1		/* define to indicate no zooming */
#define SMILEYSIZE	33		/* width and ht of smiley sprite */

/**********************************************************************/
/* declarations of routines in this demo module (not part of library) */
/**********************************************************************/
int    menu(void);
void   textdemo(void);
int    rectangledemo(void);
void   palettedemo(void);
void   pixeldemo(void);
void   linedemo(int linemode);
void   circledemo(void);
void   windowdemo(void);
void   imagedemo(void);
int    DOS_version(void);
int    check_system_requirements(void);
int    spiral_lines(int color);
void   command_line_msg(void);
double drand(void);
int    fileread(char *fname, long startbyte, long numbytes, char far *buffer);
long   filesize(char *filepath);
int    getkeypress(void);
void   draw_smiley(int x, int y, int first_draw);
long   asc_to_long(char str[]);

char *imagefilename = "s256demo.img";		/* name of demo image file */

/**************************************************************************
/* main		main routine of demo
/**************************************************************************/
void  main(int argc, char *argv[])
{
	int  modenum, choice=0, error;
	char  str[50];


	/******************************************************/
	/* check for correct number of command-line arguments */
	/******************************************************/

	if (argc < 2)
		command_line_msg();

	/*************************************************************/
	/* check to make sure this PC has all necessary requirements */
	/* 1) VGA card, 2) 80386/80486, 3) DOS 3.0 or greater        */
	/*************************************************************/

	if (check_system_requirements())
		exit(0);		/* something missing, so exit */

	/************************************************/
	/* enter graphics mode (or exit if not able to) */
	/************************************************/
	
	modenum = (int) asc_to_long(argv[1]);

	if (modenum)
		error = SV_setgmode(modenum);
	else
		command_line_msg();


	switch(error)
	{
		case 1:
		   printf("VESA driver not installed!\n"); exit(0);
		case 2:
		   printf("This graphics mode not supported!\n"); exit(0);
		case 3:
		   printf("Card not supported - video pages must be 64K bytes!\n");  exit(0);
		case 4:
		   printf("Card not supported - read segment addr must equal write segment addr!\n"); exit(0);
	}

	/**********************************************************************/
	/* use an 8x16 text font as our default text font (except in 320x200) */
	/* (available fonts are 8x8, 8x14, and 8x16) 	  		      */
	/**********************************************************************/

	SV_setfont((SV_xres() > 320) ? FONT16 : FONT8a);

	/**********************************************************************/
	/* if our screen width is greater than 800 then use a 2x zoom on text */
	/* to make actual displayed font size 16x32.			      */
	/**********************************************************************/

	SV_setcharsize((SV_xres() > 800) ? 2 : 1);


	/***********************/
	/* Lets start the demo */
	/***********************/


	while(choice != '9')		/* go until exit key pressed */
	{
		switch(choice = menu())
		{
		   case '0':
			textdemo(); break;	/* disp text */
		   case '1':
			rectangledemo(); break;	/* disp rectangles */
		   case '2':
			palettedemo(); break;	/* disp current colors */
		   case '3':
			pixeldemo(); break;	/* disp random pixels */
		   case '4':
			linedemo(PUT_MODE); break;  /* disp PUT'd lines */
		   case '5':
			linedemo(XOR_MODE); break;  /* disp XOR'd lines */
		   case '6':
			circledemo(); break;	/* disp circles and ellipses */
		   case '7':
			windowdemo(); break;	/* disp random size windows */
		   case '8':
			imagedemo(); break;	/* disp demo bit-map image */
		}
	}

	SV_freeallblocks();	/* free mem used to save screen areas (blocks)*/
	SV_freeallwinds();	/* if any open windows, free mem used by them*/
	SV_freeallhimem();      /* if any ext. memory allocated, free it */
	SV_fadeout(FADESTEPS, FADEDELAY);     /* fade the screen out */
	SV_setgmode(0x03);	/* back to text mode and we're outa here */
	SV_msecdelay(500);	/* to hide screen wavering on some monitors */
}

/***************************************************************************
/* menu		Draws menu (with random pixel background display) and returns
/*		keypress ascii code.  If enough memory is available this
/*		menu screen is only drawn once (the first time it is called)
/*		and then copied to memory by using the SV_saveblock() routine.
/*		Whenever the menu screen is again needed it is copied back
/*		to the screen using SV_closeblock() without having to draw
/*		anything.  This way the random pixels on the menu screen
/*		are never lost.  If there is not enough memory to hold the
/*		screen then the menu is redrawn from scratch each time.
/**************************************************************************/
int  menu(void)
{
	static int  not_first_time_menu, screensaved, screenblocknum;
	int  windownum1, windownum2, x, y, color, key = 0;
	char  screensizemsg[50];

	/******************************************************************/
	/* fade screen out if this is not the first 			  */
	/* time this routine is called.  Else if it is the first time     */
	/* set the "screensaved" flag to FALSE because we have not saved  */
	/* this menu screen from last time and we need to draw it for the */
	/* first time ever.  The quick SV_fadeout() primes the SV_fadein  */
	/* routine. 							  */
	/******************************************************************/

	if (not_first_time_menu)
		SV_fadeout(FADESTEPS, FADEDELAY);
	else
	{
		SV_fadeout(1, 0);		/* VERY quick fadout */
		screensaved = FALSE;
	}


	/****************************************************/
	/* either draw screen or restore it from last time. */
	/****************************************************/

	if (screensaved == FALSE)	/* menu screen not saved, so draw it */
	{
		/* clear the screen and draw border around it */
		SV_clearscreen(MENUBKG_CLR);
		SV_screenborder(BORDER_CLR);

		/* make a string that tells what size our screen is */
		sprintf(screensizemsg, "Screen is %d x %d pixels",
			SV_xres(), SV_yres());

		/* display this "screen-size" string in a window */
		x = SV_xmax()/2 - SV_stringwidth(screensizemsg)/2 - 
			SV_charwidth();
		y = (SV_xres() > 800) ? 10 : SV_charht();

		windownum1 = SV_openwind(FALSE, x, y,
			x + SV_stringwidth(screensizemsg) + SV_charwidth() * 2,
			y + 2 * SV_charht(),
			BORDER_CLR, RECTFILL_CLR, SHADOW_CLR);

		SV_setcharorigin(x, y);		/* set text origin to window */
		SV_putstringxy(screensizemsg, SV_charx(1),
			SV_chary(1) - SV_charht()/2, TEXT_CLR);
		SV_lastcharorigin();		/* restores old text origin */

		/* now display menu by opening a window */

		windownum2 = centermsg(MENU_MSG, FALSE);

		/**************************************************************/
		/* The 2 windows on the screen have just been opened.  Memory */
		/* was allocated for the data under the windows so we will    */
		/* free up this memory because we have no need to restore the */
		/* screen under the window.  Since we don't want to restore   */
		/* the data (SV_closewind() does this) we will use 	      */
		/* SV_freewind() to free the memory allocated.		      */
		/**************************************************************/

		SV_freewind(windownum2);  /* free mem of menu window */
		SV_freewind(windownum1);  /* free mem of "screen-size" window */
	}
	else
		SV_closeblock(screenblocknum);   /* restore saved menu screen */


	/********************************************/
	/* fade the screen in, using FADESTEPS steps */
	/* FADEDELAY milliseconds between each step.*/
	/********************************************/

	SV_fadein(FADESTEPS, FADEDELAY);


	/************************************************************/
	/* wait for user input and while waiting draw random pixels */
	/************************************************************/

	while((key < '0') || (key > '9'))
	{
		key = getkeypress();

		/* get random x, y, and color */
		x = drand() * (double) SV_xmax();
		y = drand() * (double) SV_ymax();
		color = drand() * 127.0;

		/* draw colored pixels on screen over background pixels */
		if (SV_readpixel(x, y) == MENUBKG_CLR)
			SV_writepixel(x, y, color);

	}


	/*****************************************************
	/* If we are using HIMEM.SYS we will use SV_saveblock()
	/* to save the whole screen to restore next time     
	/*  (instead of redrawing the screen again) 	    
	/* We will redraw the screen if there is not much  
	/* extended memory available to save the screen.  
	/* NOTE:  You will know if your screen was saved by
	/*  the random pixels on the screen.  If the pixels
	/*  cleared with every display of the main menu we 
	/*  know the screen is redrawn every time and not 
	/*  saved & restored from memory by SV_saveblock() and
	/*  SV_restoreblock().				     
	/*****************************************************/

	if (SV_himemexists() && (SV_availhimem() > (long) SV_xres()*SV_yres()))
	{
		if ((screenblocknum = SV_saveblock(0, 0, 
			SV_xmax(), SV_ymax())) != -1)
				screensaved = TRUE;
		else
				screensaved = FALSE;
	}


	/*******************************************************************/
	/* set flag for next time to show we've been through this routine. */
	/*******************************************************************/

	not_first_time_menu = TRUE;

	return key;			/* return user's key code */
}

/***************************************************************************/
/* textdemo	Demos the text display routines.
/***************************************************************************/
void textdemo(void)
{
	SV_fadeout(FADESTEPS, FADEDELAY);
	SV_clearscreen(DEMOBKG_CLR);
	SV_screenborder(BORDER_CLR);
	SV_fadein(FADESTEPS, FADEDELAY);

	/********************************************************************
	/* save the current font, zoom size, and origin of the char routines.
	/* This "SV_savecharstate()" works like a stack with 
	/*"SV_restorecharstate()".  (pushes & pops states)
	/*********************************************************************/

	SV_savecharstate();


	/*******************************************************************/
	/* set all text rows and cols as offsets from pixel coordinates 0,0*/
	/*******************************************************************/

	SV_setcharorigin(0, 0);


	/******************************************************/
	/* if 1024x768 mode then start with a 2x zoom on text */
	/******************************************************/

	if (SV_xres() > 800)
		SV_setcharsize(2);		/* 2x zoom */
	else
		SV_setcharsize(1);		/* no zoom */


	/****************************************************************
	/* about color values:
	/* Bits 0-7 define the foreground pixel color, bits 8-15 define
	/* the background pixel color.  Combine these 2 colors to get
	/* a 16 bit color value.  If background bits are zero then the
	/* background pixels are not drawn (this is faster).
	/****************************************************************/

	/* 1st we display a string without drawing the text 	*/
	/* background pixels.  TEXT_CLR is an 8-bit color value. */

	SV_putstringxy("Text without background pixels drawn, press key to continue",
		SV_charx(1), SV_chary(1), TEXT_CLR);	getch();


	/**************************************************************
	/* next we display a string with background pixels drawn.     
	/* The macro SV_BF (found in svga256.h) forms the 16-bit color
	/* value from our 8-bit foreground color and 8-bit bkg color.
	/**************************************************************/

	SV_putstringxy("Text with background pixels drawn, press key to continue   ", SV_charx(1), SV_chary(1), SV_BF(MENUBKG_CLR, TEXT_CLR));  

	getch();

	/* clear this string (we actually clear whole character row here) */
	SV_movetoxy(SV_charx(1), SV_chary(1));
	SV_clearstring(SV_charmaxcol() - 1, DEMOBKG_CLR);



	/*********************************************************
	/* Now we will show the different fonts and zoom factors.
	/**********************************************************/

	/* set color and charsize (zoom factor) to no zoom */

	SV_setcharsize(1);
	SV_setcolor(TEXT_CLR);

	/* first show the ROM BIOS 8x8 font lower 128 characters.  */
	/* (FONT8b is the upper 128 characters - special chars)    */

	SV_setfont(FONT8a);
	SV_putstringxy("This the 8x8 font found in your ROM BIOS",
		SV_charx(1), SV_chary(1), TEXT_CLR);



	/* now show 8x14 font (all 255 characters available with this font) */

	SV_setfont(FONT14);
	SV_putstringxy("This the 8x14 font found in your ROM BIOS",
		SV_charx(1), SV_chary(2), TEXT_CLR);



	/* use a 8x16 font for the rest of demo (all 255 chars available */

	SV_setfont(FONT16);
	SV_putstringxy("This the 8x16 font found in your ROM BIOS",
		SV_charx(1), SV_chary(4), TEXT_CLR);



	/* move down a couple char rows - char column 0, char row 4 */
	/* and show the zooming effect of "SV_setcharsize()"	    */

	SV_movetoxy(SV_charx(1), SV_chary(5));
	SV_setcharsize(1);
	SV_putstring("8 x 16 font with no zoom, ");
	SV_setcharsize(2);
	SV_putstring("or 2 zoom, ");
	SV_setcharsize(3);
	SV_putstring("or 3 zoom, ");
	SV_movetoxy(SV_charwidth(), SV_gety() + SV_charht());
	SV_setcharsize(4);
	SV_putstring("or 4, ");
	SV_setcharsize(5);
	SV_putstring("or 5, ...");
	SV_setcharsize(2);



	/****************************************************************/
	/* set font, font size, char origin coordinates back to what it */
	/* was before the above call to SV_savecharstate().		*/
	/* SV_savecharstate() and SV_restorecharstate() work like a     */
	/* stack and calls to the two should be balanced.		*/	
	/****************************************************************/

	SV_restorecharstate();


	/****************************************************************/
	/* display a message window in center of screen instructing user */
	/* to press a key to get the main menu back.			 */
	/****************************************************************/

	centermsg(PRESS_MSG, TRUE);
}

/**************************************************************************
/* rectangledemo	Draw concentric rectangles in differing colors and
/*			show the "SV_fillrect" and "SV_framedrect" routines.
/**************************************************************************/
int rectangledemo(void)
{
	static char *string1 = "SV_fillrect()";
	static char *string2 = "SV_framedrect()";
	int  x1, y1, x2, y2, i, color=1;

	SV_fadeout(FADESTEPS, FADEDELAY);
	SV_clearscreen(DEMOBKG_CLR);
	SV_screenborder(BORDER_CLR);
	SV_fadein(FADESTEPS, FADEDELAY);


	/********************************************************************
	/* save the current font, zoom size, and origin of the char routines.
	/* This "SV_savecharstate()" works like a stack (pushes & pops states)
	/* with "SV_restorecharstate()".
	/********************************************************************/

	SV_savecharstate();


	/******************************/
	/* draw concentric rectangles */
	/******************************/

	x1 = 0; y1 = 0; x2 = SV_xmax(); y2 = SV_ymax();

	while((x1 < x2) && (y1 < y2))
	{
		SV_rectangle(x1, y1, x2, y2, color);
		color = (color + 1) % 256;
		x1+=4; y1+=4; x2-=4; y2-=4;
	}


	/**********************************************/
	/*  draw a filled rectangle containing string1*/
	/**********************************************/

	x1 = SV_xmax()/10;
	y1 = SV_ymax()/10;
	x2 = x1 + SV_stringwidth(string1) + 2 * SV_charwidth();
	y2 = y1 + 3 * SV_charht();

	SV_fillrect(x1, y1, x2, y2, RECTFILL_CLR);
	SV_setcharorigin(x1, y1);
	SV_putstringxy(string1, SV_charx(1), SV_chary(1), TEXT_CLR);


	/***********************************************/
	/*  draw a framed rectangle containing string2 */
	/***********************************************/

	x1 = SV_xmax() - SV_stringwidth(string2) * 2;
	y1 = SV_ymax() - 5 * SV_charht();
	x2 = x1 + SV_stringwidth(string2) + 2 * SV_charwidth();
	y2 = y1 + 3 * SV_charht();

	SV_framedrect(x1, y1, x2, y2, BORDER_CLR, RECTFILL_CLR);
	SV_setcharorigin(x1, y1);
	SV_putstringxy(string2, SV_charx(1), SV_chary(1), TEXT_CLR);


	/****************************************************************/
	/* set font, font size, char origin coordinates back to what it */
	/* was before the above call to SV_savecharstate().		*/
	/****************************************************************/

	SV_restorecharstate();

	centermsg(PRESS_MSG, TRUE);	/* display msg, wait for key press */

	return 0;
}

/****************************************************************************
/* palettedemo		Displays all 255 colors and their number.
/****************************************************************************/
void  palettedemo(void)
{
	int  i, j, x, y, lt, top, rt, bot, colorlabel, areaht;
	int   color, templt, width, ht, areawidth;
	char  tempstr[10];
	static char  *presskeymsg = "Press key for main menu.";


	SV_fadeout(FADESTEPS, FADEDELAY);
	SV_clearscreen(DEMOBKG_CLR);
	SV_screenborder(BORDER_CLR);
	SV_fadein(FADESTEPS, FADEDELAY);

	SV_savecharstate();

	areawidth = (float) SV_xmax() / 1.1;
	areaht = (float) SV_ymax() / 1.2;

	/* handle special case with low-res 320x200 mode */
	if (SV_xres() == 320)
		areaht = (float) SV_ymax() / 1.5;

	lt = SV_xmax()/2 - areawidth/2;
	top = SV_ymax()/2 - areaht/2;
	rt = lt + areawidth - 1;
	bot = top + areaht - 1;
	width = (rt - lt + 1) / 16;
	ht = (bot - top + 1) / 16;

	SV_setfont(FONT16);
	SV_setcharsize(1);

	SV_fillrect(lt, top, rt, bot+SV_charht()*2, BORDER_CLR);

	if (SV_xres() > 800) SV_setcharsize(2);
	x = rt + (lt - rt)/2 - SV_stringwidth(presskeymsg)/2;
	y = bot;
	SV_putstringxy(presskeymsg, x, y, RECTFILL_CLR);
	SV_setcharsize(1);

	color = 0;
	colorlabel = 0;

	for(i = 0; i < 16; i++)
	{
		templt = lt;

		for(j = 0; j < 16; j++)
		{
			SV_framedrect(templt, top, templt + width - 1,
				top + ht - 1, BORDER_CLR, color);

			if (SV_xres() > 320) /* skip numbers for 320x200 mode */
			{
			    sprintf(tempstr, "%d", colorlabel++);
			    SV_putstringxy(tempstr, templt + 3, 
					top + 3, BORDER_CLR);
			}

			color++;
			templt += width;
		}

		top += ht;
	}

	SV_restorecharstate();		/* restore the old text parameters */

	while(!getkeypress());             /* wait for key press */
}

/***************************************************************************
/* pixeldemo   Draw randomly positioned pixels using colors
/*		0 - 127 (default palette has brighter colors in this range).
/*		until a key is pressed.
/*		Uses drand(), a random number generator.
/**************************************************************************/
void  pixeldemo(void)
{
	int  x, y, color, smiley_y;
	long ticks;
	static int smiley_x = 0;


	SV_fadeout(FADESTEPS, FADEDELAY);
	SV_clearscreen(DEMOBKG_CLR);
	SV_screenborder(BORDER_CLR);
	SV_fadein(FADESTEPS, FADEDELAY);

	smiley_y = SV_ymax() / 2;
	draw_smiley(smiley_x, smiley_y, 1);

	while(!getkeypress())
	{
		x = drand() * (double) SV_xmax();
		y = drand() * (double) SV_ymax();
		color = drand() * 127.0;

		SV_writepixel(x, y, color);

		/* change smiley position with each clock tick */
		if (TICKS != ticks)
		{
			smiley_x += 2; 
			smiley_x %= (SV_xmax() - SMILEYSIZE);
			draw_smiley(smiley_x, smiley_y, 0);
			ticks = TICKS;
		}
	}
}

/***************************************************************************
/* linedemo   Draw randomly positioned lines with different widths and styles
/*	      using colors 0 - 127 until a key is pressed.
/*	      If "linemode" is 0 lines are PUT onto the screen.  If
/*	      "linemode" is 1 the lines are XOR'd onto the screen twice
/*	       which has the effect of display-and-erase.
/**************************************************************************/
void  linedemo(int linemode)
{
	int  x1, y1, x2, y2;
	int  xradius, color, linewidth, linestyle;
	int  savedlinewidth, savedlinestyle, savedlinemode;
	float aspect;


	SV_fadeout(FADESTEPS, FADEDELAY);
	SV_clearscreen(DEMOBKG_CLR);
	SV_screenborder(BORDER_CLR);
	SV_fadein(FADESTEPS, FADEDELAY);

	/* force linemode 0 or 1 (PUT_MODE or XOR_MODE) */
	linemode &= 1;


	/* save the current line parameters to restore them when done */
	savedlinestyle = SV_getlinestyle();
	savedlinewidth = SV_getlinewidth();
	savedlinemode  = SV_getlinemode();

	/* put something pretty on the screen to draw over */
	SV_fillcircle(SV_xmax()/2, SV_ymax()/2, SV_ymax()/3, SUN_CLR);

	SV_setlinemode(linemode); 		/* PUT or XOR */

	while(!getkeypress())
	{
		x1 = drand() * (double) SV_xmax();
		y1 = drand() * (double) SV_ymax();
		x2 = drand() * (double) SV_ymax();
		y2 = drand() * (double) SV_ymax();
		color = drand() * 127.0;
		linewidth = (int) (drand() * 7.1) & 0xFE;
		linestyle = (int) (drand() * 7.1);
		SV_setlinewidth(linewidth);
		SV_setlinestyle(linestyle);

		SV_line(x1, y1, x2, y2, color);
		SV_msecdelay((linemode) ? 600 : 200);
		if (linemode)
			SV_line(x1, y1, x2, y2, color);
	}

	/* restore old line-drawing parameters */
	SV_setlinewidth(savedlinewidth);
	SV_setlinestyle(savedlinestyle);
	SV_setlinemode(savedlinemode);
}

/***************************************************************************
/* circledemo   Draw randomly positioned circles and ellipses (filled and
/*		not filled) using colors 0 - 127 until a key is pressed.
/**************************************************************************/
void  circledemo(void)
{
	int  x, y, r;
	int  xradius, color, savedlinewidth, linewidth;
	float aspect;
	long ticks;


	SV_fadeout(FADESTEPS, FADEDELAY);
	SV_clearscreen(DEMOBKG_CLR);
	SV_screenborder(BORDER_CLR);
	SV_fadein(FADESTEPS, FADEDELAY);

	savedlinewidth = SV_getlinewidth();

	while(!getkeypress())
	{
		x = drand() * (double) SV_xmax();
		y = drand() * (double) SV_ymax();
		r = drand() * (double) SV_ymax();
		aspect = (float) drand() + 0.5;
		color = drand() * 127.0;
		linewidth = (int) (drand() * 7.1) & 0xFE;
		SV_setlinewidth(linewidth);


		/* draw an ellipse 1/4 of the time, the rest circles */
		if ((x % 4) == 0)
		{
			/* calculate "width" of ellipse */
			xradius = (float) r / aspect;

			/* draw only if within screen boundary */
			if ((x-xradius-linewidth > 0) &&
			    (x+xradius+linewidth < SV_xmax()) &&
			    (y-r-linewidth > 0) &&
			    (y+r+linewidth < SV_ymax()))
			{
				/* get even mix of filled, nonfilled ellipses*/
				if (y & 1)
					SV_ellipse(x, y, r, color, aspect);
				else
					SV_fillellipse(x, y, r, color, aspect);
			}
		}
		else			/* circles */
		{
			/* draw only if within screen boundary */
			if ((x-r-linewidth > 0) &&
			    (x+r+linewidth < SV_xmax()) &&
			    (y-r-linewidth > 0) &&
			    (y+r+linewidth < SV_ymax()))
			{
				/* get even mix of filled, nonfilled circles*/
				if (y & 1)
					SV_circle(x, y, r, color);
				else
					SV_fillcircle(x, y, r, color);
			}
		}
	}

	SV_setlinewidth(savedlinewidth);
}


/*****************************************************************************
/* windowdemo		Display randomly positioned, randomly sized, randomly
/*			colored "pop-up" windows one after the other.
/*****************************************************************************/
void  windowdemo(void)
{
	int  windownum, x, y, width, ht, borderclr, fillclr, shadowclr;
	int  savedlinewidth;
	WORD ct=0;


	SV_fadeout(FADESTEPS, FADEDELAY);
	SV_clearscreen(DEMOBKG_CLR);
	SV_screenborder(BORDER_CLR);
	SV_fadein(FADESTEPS, FADEDELAY);

	savedlinewidth = SV_getlinewidth();

	spiral_lines(BORDER_CLR);

	/*****************************************************************/
	/* lets open a message window instructing user to press key for  */
	/* main menu.  Then lets free the memory allocated for the saved */
	/* screen info under this window since we will not need to       */
	/* restore this data, because we will redraw the whole screen    */
	/* anyways when we go back to the menu screen.			 */
	/*****************************************************************/

	windownum = centermsg(PRESS_MSG, FALSE);
	SV_freewind(windownum);


	/***************************************************************
	/* start the randomly colored, randomly sized window opening and 
	/* closing (SV_closewind() restores the screen data, SV_freewind() is
	/* the same as SV_closewind() except it doesn't restore the screen.
	/* We make the shadowclr -1 about every 4th window which tells  
	/* SV_openwind() to skip the shadow. 
	/* Every 3rd window is saved on the screen.	
	/*****************************************************************/

	while(!getkeypress())
	{
		x = drand() * (double) SV_xmax();
		y = drand() * (double) SV_ymax();
		width = drand() * (double) SV_xmax()/1.2 + 10.0;
		ht = drand() * (double) SV_ymax()/1.2 + 10.0;
		borderclr = drand() * 127.0;
		fillclr = drand() * 127.0;
		shadowclr = ((fillclr % 4) == 0) ? -1 : drand() * 127.0;
		SV_setlinewidth((int) (drand() * 9.0));

		windownum = SV_openwind(FALSE, x, y, x + width, y + ht,
			borderclr, fillclr, shadowclr);

		/* window numbers will be > 0 if the */
		/* window is successfully opened.     */

		if (windownum > 0) 
		{
			ct++;

			if ((ct % 3) == 0)
			{
				SV_hidewind(windownum);
				SV_msecdelay(100);

				SV_unhidewind(windownum);
				SV_msecdelay(500);

				SV_freewind(windownum);
			}
			else
			{
				SV_msecdelay(200);
				SV_closewind(windownum);
			}
		}
	}

	SV_setlinewidth(savedlinewidth);
}

/***************************************************************************
/* imagedemo	Display a bit-mapped image stored on disk.  This file has
/*		the pixel data in uncompressed form with the 1st byte of
/*		the file representing the 1st pixel of the image.  The image
/*		is a grayscale image with pixel values from 128-255 so we
/*		must set the palette to have a grayscale in this range.  We
/*		fit a 64-level grayscale in this range to display
/*		the image.
/***************************************************************************/
void  imagedemo(void)
{
	BYTE  far *image, savedpalette[DACREGS];
	int   x, y, oldx, oldy, zoom, windownum;
	FILE  *fptr;
	long  fsize;


	SV_fadeout(FADESTEPS, FADEDELAY);
	SV_clearscreen(DEMOBKG_CLR);
	SV_screenborder(BORDER_CLR);
	SV_fadein(FADESTEPS, FADEDELAY);

	spiral_lines(SUN_CLR);

	/* if using 320x200 mode then skip this demo, image is too big */
	if (SV_xres() == 320)
	{
		SV_freewind(centermsg(MODE320x200_MSG, TRUE));
		return;
	}

	windownum = centermsg(READ_MSG, FALSE);
	SV_freewind(windownum);

	/*********************/
	/* get the file size */
	/*********************/
	fsize = filesize(imagefilename);

	if (fsize == 0)
	{
		centermsg(FILEERR_MSG, TRUE);
		return;
	}


	/*******************************/
	/* alloc space to read in file  /
	/*******************************/
	if ((image = (BYTE far *) farmalloc(fsize)) == 0)
	{
		centermsg(ALLOCERR_MSG, TRUE);
		return;
	}


	/**************************/
	/* read in the demo image */ 
	/**************************/
	if (fileread(imagefilename, 0L, fsize, image) == -1)
	{
		centermsg(FILEERR_MSG, TRUE);
		return;
	}


	/*****************************************************************
	/* save the current 256 colors and then make colors 128-255 a
	/* grayscale of 64 different grays where 128 = BLACK, 255 = WHITE
	/* (This image has pixel values between 128-255).
	/*****************************************************************/
	SV_readDAC(0, 256, savedpalette);
	SV_initgrayscale64(2, 0);



	/*************************************************************
	/* calculate the image left-upper corner coordinates (x,y),
	/* display image (1x zoom factor), then draw border around image.
	/*************************************************************/
	x = SV_xmax()/2 - IMAGEWIDTH/2;
	y = SV_ymax()/2 - IMAGEHT/2;
	SV_putimage(image, x, y, IMAGEWIDTH, IMAGEHT, 1);
	SV_rectangle(x, y, x + IMAGEWIDTH - 1, y + IMAGEHT - 1, BORDER_CLR);


	/**********************************************************/
	/* put some text description in image, and wait for a key */
	/* press before continuing with demonstration.            */
	/**********************************************************/
	SV_savecharstate();	/* save current font, char zoom size, origin */

	SV_setfont(FONT16);	/* use 8x16 font */
	SV_setcharsize(1);         /* use normal size (no zoom) on text */
	SV_setcharorigin(x+2, y+2);  /* set text origin to just inside image */

	/* display text */
	SV_putstringxy("MEDICAL ULTRASOUND IMAGE", 
			SV_charx(0), SV_chary(0), TEXT_CLR);
	SV_movetoxy(SV_charx(0), SV_chary(1)); SV_setcolor(TEXT_CLR);
	SV_putstring("64 gray levels, 501x333 pixels (unzoomed).");

	SV_restorecharstate();	/* restore old font, char zoom size, origin */

	while(!getkeypress());		/* wait for a key press */


	/*********************************************************/
	/* Now demonstrate moving the image to random positions  */
	/*********************************************************/
	while(!getkeypress())		/* loop till key is pressed */
	{
		oldx = x; oldy = y;
		x = drand() * (double) (SV_xmax() - IMAGEWIDTH - 1);
		y = drand() * (double) (SV_ymax() - IMAGEHT - 1);
		x++; y++;	/* avoid corrupting screen border */
		SV_moveimage(oldx, oldy, x, y, IMAGEWIDTH, IMAGEHT);
	}


	/***************************************************************/
	/* Now if screen is large enough lets zoom the image (2x zoom) */
	/***************************************************************/
	if ((SV_xres() > IMAGEWIDTH * 2) && (SV_yres() > IMAGEHT * 2))
	{
		/* first lets "get" the image from the screen (now that  */
		/* it has text in it) so we don't have to redraw the     */
		/* text and border.    */
		SV_getimage(image, x, y, IMAGEWIDTH, IMAGEHT);

		x = SV_xmax()/2 - IMAGEWIDTH;	/* get center screen x */
		y = SV_ymax()/2 - IMAGEHT;         /* get center screen y */
		SV_putimage(image, x, y, IMAGEWIDTH, IMAGEHT, 2);
		while(!getkeypress());		/* wait for a key press */
	}


	/********************************************************/
	/* free the image memory and restore the old 256 colors */
	/********************************************************/

	farfree(image);		/* free memory used to hold image data */
	SV_writeDAC(0, 256, savedpalette); /* restore old colrs (no grayscale)*/
}




/*mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
 *mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
 *mmmmmm The rest of these routines are utils for the above demos. mmmmmmmmm
 *mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
/*mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm*/

/***************************************************************************/
/* asc_to_long:  Converts a string of ascii digits to a long integer value.*/
/*		 The string can hold a hex-number string but hex values    */
/*		 must start with "0x" (i.e. "0x4B33F") otherwise it is     */
/*		 assumed the string represents a decimal number.   	   */
/***************************************************************************/
long  asc_to_long(char str[])
{
	char  i = 0, ch, start_i, base_value;
	long  mult = 1, sum = 0;

	if (!str[i]) return 0;			/* empty string! */

	while (str[i] == '0') i++;		/* skip leading zeros */

	if ((i > 0) && (str[i] == 'x'))		/* check for hex number */
	{	
		base_value = 16;
		start_i = ++i;
	}
	else					/* must be decimal */
	{ 	
		base_value = 10;
		start_i = i;
	}

	while (str[i+1]) i++;                 /* go to end of str */

	while (i >= start_i)			/* process digits        */
	{					/* backwards in string. */
		ch = str[i--];

		if ((ch >= '0') && (ch <= '9'))
			ch = ch - '0';
		else if (((ch & 95) >= 'A') && ((ch & 95) <= 'F') &&
		    (base_value == 16))
			ch = (ch & 95) - 'A' + 10;
		else
			return 0;		/* error */

		sum += mult * ch;

		mult *= base_value;
	}

	return (str[0] == '-') ? -sum : sum;
}

/**************************************************************************
/* spiral_lines		Draws a "spiral" of lines on the screen.
/*			Used as a backdrop in windowdemo() and imagedemo().
/**************************************************************************/
int  spiral_lines(int color)
{
#define INCRS	100
#define BORD	25

	int  x1, y1, x2, y2, xinc;

	x1 = BORD; y1 = BORD; x2 = SV_xmax()-BORD; y2 = SV_ymax()-BORD;
	xinc = (SV_xmax()-BORD) / INCRS;

	while(x2 >= BORD)
	{
		SV_line(x1, y1, x2, y2, color);
		x1 += xinc;
		x2 -= xinc;
	}

	x1 = BORD; y1 = BORD; x2 = SV_xmax()-BORD; y2 = SV_ymax()-BORD;
	xinc = (SV_yres()-BORD) / INCRS;
	while(y2 >= BORD)
	{
		SV_line(x1, y1, x2, y2, color);
		y1 += xinc;
		y2 -= xinc;
	}

	return 0;
}

/**************************************************************************
/* check_system_requirements:	this routine checks
/*	for all the requirements of this PC for this program to run. If
/*	a requirement is not met then a message is printed and TRUE
/*	is returned.  If all requirements are met FALSE is returned.
/*	Requirements:
/*		1) 80386 or greater CPU
/*		2) VGA video display adapter
/*		3) DOS 3.0 or greater
/**************************************************************************/
int  check_system_requirements(void)
{
	int  graphdriver, graphmode, error = FALSE;


	if (SV_getprocessor() != P_80386)
	{
		printf("ERROR! THIS PROGRAM REQUIRES AN 80386 PROCESSOR.\n");
		error = TRUE;
	}


	if (!SV_vgacard())
	{
		printf("ERROR! THIS PROGRAM REQUIRES A VGA VIDEO ADAPTER.\n");
		error = TRUE;
	}

	if (DOS_version() < 3)
	{
		printf("ERROR! THIS PROGRAM REQUIRES DOS VERSION 3.0 OR GREATER.\n");
		error = TRUE;
	}

/*************************************************************************** 
    May  also want to check for himem.sys, although this graphics
    library does not require himem.sys to run.  Needs himem.sys only if
    you want to use the extended memory routines or open large windows
    using extended memory. 
/***************************************************************************/

/*
	if (!SV_himemexists())
	{
		printf("ERROR! PROGRAM REQUIRES HIMEM.SYS TO BE INSTALLED!\n");
		error = TRUE;
	}
*/

	return error;
}

/***************************************************************************
/* DOS_version:		Returns the whole number part of the DOS version
/*			currently being used. (i.e.  Returns 3 for DOS 3.2)
/***************************************************************************/
int  DOS_version(void)
{
	union REGS regs;

	regs.h.ah = 0x30;
	intdos(&regs, &regs);
	return (int) regs.h.al;
}

/*************************************************************************
/* drand()	Linear congruential random number generator
/*		Returns a random number between 0.001 and 1.0.
/*************************************************************************/
double  rm = 16777216.0, ra = 314159.0, rc = 271828.0, ri = 577215.0;
double drand(void)
{
	double  rx, rnn, rrn;

	do
	{
		rx = ra * ri + rc;
		rnn = rx - rm * (double)((long) (rx / rm));
		rrn = rnn / rm;
		ri = rnn;
	}
	while(rrn < 0.001);	/* reject numbers < 0.001 */

	return  rrn;
}


/***************************************************************************
/* command_line_msg	Prints info about s256demo.exe and then exits program.
/***************************************************************************/
void command_line_msg()
{
	int  i;

	for(i = 0; i <15; i++) printf("\n");  /* clear the screen */

	printf("\n     S256DEMO demonstrates the use of the SVGA256 Graphics Library\n");
	printf(" to draw and manipulate graphics displays in the 256-color Super VGA\n");
	printf(" modes.  You must have a VESA compatible video card or have your card's\n");
	printf(" VESA driver installed (check your card's manual or look at the files\n");
	printf(" that came with the card).  The VESA 256-color graphic modes are:\n\n");
	printf("       256 (0x100) for the 640x400 pixel resolution\n");
	printf("       257 (0x101) for the 640x480 pixel resolution\n");
	printf("       259 (0x103) for the 800x600 pixel resolution\n");
	printf("       261 (0x105) for the 1024x768 pixel resolution\n");
	printf("       263 (0x107) for the 1280x1024 pixel resolution (untested)\n\n");
	printf(" One of these mode numbers must be supplied to S256DEMO as a command-line\n");
	printf(" argument. If your card has a TSENG 4000 video processor on it you can\n");
	printf(" use the TSENG mode numbers 46, 47, 48 & 56 without VESA compatibility.\n");
	printf(" The library also works with the standard VGA mode 19 (13 hex).\n\n");
	printf("           Command-Line Syntax:  S256DEMO mode#\n");
	exit(1);
}


/******************************************************************************
/* fileread	Opens file 'fname' for reading, reads 'numbytes' bytes of
/*		data from the file starting with byte number 'startbyte'.
/*		Reads data into 'buffer'.
/*
/*		Error checks:
/*			1) opening of stream (fopen).
/*			2) fptr positioning (fseek).
/*			3) file read (fread).
/*
/*		Returns -1 on error, else 0.
/*****************************************************************************/
int  fileread(char *fname, long startbyte, long numbytes, char far *buffer)
{
#define MAX_FILEIO_SIZE	65024

	FILE  *fptr = 0;
	char  huge *hugeptr;
	long  readsize, bytesleft;
	int   error = 0;


	/* initialize variables for this procedure */
	bytesleft = numbytes;
	hugeptr = (char huge *) buffer;

	/* open the file for reading */
	if ((fptr = fopen(fname, "rb")) == 0)
		error = -1;

	/* position the file pointer to starting read position */
	else if (startbyte && (fseek(fptr, startbyte, 0) != 0))
		error = -1;

	while (!error && bytesleft)
	{
		readsize = (bytesleft >= MAX_FILEIO_SIZE) ? 
				MAX_FILEIO_SIZE : bytesleft;

		if (fread((void *) buffer, 1, readsize, fptr) != readsize)
			error = -1;

		hugeptr += readsize;
		buffer = (char far *) hugeptr;
		bytesleft -= readsize;
	}

	if (fptr != 0) fclose(fptr);

	return error;
}

/***************************************************************************
/* filesize	Returns the size in bytes of a file on disk, else 0 on error.
/*		Uses the "fopen()/fseek()/ftell()" method.
/**************************************************************************/
long  filesize(char *filepath)
{
	int   error = 0;
	long  fsize;
	FILE  *fptr;

	if ((fptr = fopen(filepath, "rb")) == 0)
		error = 1;
	else if (fseek(fptr, 0, SEEK_END) != 0)
		error = 1;
	else
		fsize = ftell(fptr);

	if (fptr) fclose(fptr);

	return (error) ? 0L : fsize;
}

/*************************************************************************
/* getkeypress	Returns key if one was pressed else returns 0
/*		Does not wait for a key press!
/*************************************************************************/
int  getkeypress(void)
{
	int key;

	if (kbhit())
	{
		key = getch();
		if (key == 0) key = (getch() | 128);
		return key;
	}

	return 0;
}

BYTE smileypattern[SMILEYSIZE][SMILEYSIZE] =
 {
	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},

	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},

	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},

	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},

	{0x00, 0x00, 0x00, 0x00, 0x00, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},

	{0x00, 0x00, 0x00, 0x00, 0x00, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x00, 0x00, 0x00, 0x00},

	{0x00, 0x00, 0x00, 0x00, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x00, 0x00, 0x00, 0x00},

	{0x00, 0x00, 0x00, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x00, 0x00, 0x00},

	{0x00, 0x00, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x1E,
	0x1E, 0x1E, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x1E, 0x1E,
	0x1E, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x00, 0x00},

	{0x00, 0x00, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x1E, 0x1E,
	0x1E, 0x1E, 0x1E, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x1E, 0x1E, 0x1E,
	0x1E, 0x1E, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x00, 0x00},

	{0x00, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x1E, 0x1E, 0x1E,
	0x1E, 0x1E, 0x1E, 0x1E, 0x2A, 0x2A, 0x2A, 0x1E, 0x1E, 0x1E, 0x1E,
	0x1E, 0x1E, 0x1E, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x00, 0x00},

	{0x00, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x1E, 0x1E, 0x1E,
	0x1E, 0x10, 0x10, 0x10, 0x2A, 0x2A, 0x2A, 0x1E, 0x1E, 0x1E, 0x1E,
	0x10, 0x10, 0x10, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x00},

	{0x00, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x1E, 0x1E, 0x1E,
	0x10, 0x10, 0x10, 0x10, 0x2A, 0x2A, 0x2A, 0x1E, 0x1E, 0x1E, 0x10,
	0x10, 0x10, 0x10, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x00},

	{0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x1E, 0x10,
	0x10, 0x10, 0x10, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x1E, 0x10, 0x10,
	0x10, 0x10, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x00},

	{0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x1E, 0x10,
	0x10, 0x10, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x1E, 0x10, 0x10,
	0x10, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A},

	{0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A},

	{0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A},

	{0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x6F, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x6F, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A},

	{0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x6F, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x6F, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A},

	{0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x6F, 0x6F, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x6F, 0x6F, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A},

	{0x00, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x6F, 0x6F, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x6F, 0x6F, 0x2A, 0x2A, 0x2A, 0x2A, 0x00},

	{0x00, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x6F, 0x6F, 0x6F, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x6F, 0x6F, 0x6F, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x00},

	{0x00, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x6F, 0x6F, 0x6F, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x6F, 0x6F, 0x6F, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x00},

	{0x00, 0x00, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x6F, 0x6F, 0x6F, 0x6F,
	0x6F, 0x6F, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x6F, 0x6F,
	0x6F, 0x6F, 0x6F, 0x6F, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x00, 0x00},

	{0x00, 0x00, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x6F, 0x6F, 0x6F,
	0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x1E, 0x1E, 0x6F, 0x6F, 0x6F, 0x6F,
	0x6F, 0x6F, 0x6F, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x00, 0x00},

	{0x00, 0x00, 0x00, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x6F, 0x6F,
	0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x1E, 0x1E, 0x6F, 0x6F, 0x6F, 0x6F,
	0x6F, 0x6F, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x00, 0x00, 0x00},

	{0x00, 0x00, 0x00, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x6F,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x00, 0x00, 0x00, 0x00},

	{0x00, 0x00, 0x00, 0x00, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x00, 0x00, 0x00, 0x00},

	{0x00, 0x00, 0x00, 0x00, 0x00, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x00, 0x00, 0x00, 0x00, 0x00},

	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},

	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x2A, 0x2A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},

	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x2A,
	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A,
	0x2A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},

	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
};

BYTE screendata_buffer[SMILEYSIZE * SMILEYSIZE];

/******************************************************************************
/* draw_smiley	Draws and erases the smiley byte pattern above using the XOR
/*		pixel drawing method of "SV_putbytepattern".  See also 
/*		"SV_putbitpattern" for sprite display. 
/*****************************************************************************/
void draw_smiley(int x, int y, int first_draw)
{
	static int oldx, oldy;
	
	/* restore screen area where smiley now is */
	if (!first_draw)
		SV_putimage((BYTE *) screendata_buffer, oldx, oldy, 
			SMILEYSIZE, SMILEYSIZE, 1);

	/* get screen area where new smiley will be drawn */
	SV_getimage(screendata_buffer, x, y, SMILEYSIZE, SMILEYSIZE);

	/* draw new smiley */
	SV_putbytepattern((BYTE *) smileypattern, x, y,
		SMILEYSIZE, SMILEYSIZE, PUT_MODE);

	/* save new smiley's coordinates for next time */
	oldx = x; oldy = y;
}


